@@ -106,7 +106,7 @@ checkout (int argc, char **argv)
     int local = 0;
     int shorten = -1;
     char *where = NULL;
-    char *valid_options;
+    const char *valid_options;
     const char *const *valid_usage;
 
     /* initialize static options */
@@ -339,7 +339,7 @@ checkout (int argc, char **argv)
 	    free (options);
 	    options = NULL;
 	}
-	return (0);
+	return 0;
     }
     db = open_module ();
 
@@ -373,7 +373,7 @@ checkout (int argc, char **argv)
     for (i = 0; i < argc; i++)
 	err += do_module (db, argv[i], m_type, "Updating", checkout_proc,
 			  where, shorten, local, run_module_prog, !pipeout,
-			  (char *) NULL);
+			  NULL);
     close_module (db);
     if (options)
     {
@@ -382,9 +382,11 @@ checkout (int argc, char **argv)
     }
     if (history_name != tag && history_name != date && history_name != NULL)
 	free (history_name);
-    return (err);
+    return err;
 }
 
+
+
 /* FIXME: This is and emptydir_name are in checkout.c for historical
    reasons, probably want to move them.  */
 
@@ -418,20 +420,20 @@ safe_location (char *where)
     size_t hardpath_len;
     int retval;
 
-    TRACE( TRACE_FUNCTION, "safe_location( where=%s )",
-           where ? where : "(null)" );
+    TRACE (TRACE_FUNCTION, "safe_location( where=%s )",
+           where ? where : "(null)");
 
 #ifdef CLIENT_SUPPORT
     /* Don't compare remote CVSROOTs to our destination directory. */
-    if ( current_parsed_root->isremote ) return 1;
+    if (current_parsed_root->isremote) return 1;
 #endif /* CLIENT_SUPPORT */
 
     /* set current - even if where is set we'll need to cd back... */
-    current = xgetwd ();
+    current = xgetcwd ();
     if (current == NULL)
 	error (1, errno, "could not get working directory");
 
-    hardpath = xresolvepath ( current_parsed_root->directory );
+    hardpath = xresolvepath (current_parsed_root->directory);
 
     /* if where is set, set current to as much of where as exists,
      * or fail.
@@ -445,7 +447,7 @@ safe_location (char *where)
 	    {
 		/* where */
 		free (where_this_pass);
-		where_this_pass = xgetwd();
+		where_this_pass = xgetcwd ();
 		if (where_this_pass == NULL)
 		    error (1, errno, "could not get working directory");
 
@@ -510,10 +512,12 @@ could not change directory to requested checkout directory `%s'",
     else
 	retval = 1;
     free (current);
-    free( hardpath );
+    free (hardpath);
     return retval;
 }
 
+
+
 struct dir_to_build
 {
     /* What to put in CVS/Repository.  */
@@ -524,11 +528,11 @@ struct dir_to_build
     struct dir_to_build *next;
 };
 
+
+
 static int build_dirs_and_chdir (struct dir_to_build *list,
 					int sticky);
 
-static void build_one_dir (char *, char *, int);
-
 static void
 build_one_dir (char *repository, char *dirpath, int sticky)
 {
@@ -546,8 +550,8 @@ build_one_dir (char *repository, char *dirpath, int sticky)
 	    error (1, 0, "there is no repository %s", repository);
 
 	if (Create_Admin (".", dirpath, repository,
-			  sticky ? tag : (char *) NULL,
-			  sticky ? date : (char *) NULL,
+			  sticky ? tag : NULL,
+			  sticky ? date : NULL,
 
 			  /* FIXME?  This is a guess.  If it is important
 			     for nonbranch to be set correctly here I
@@ -571,12 +575,16 @@ build_one_dir (char *repository, char *dirpath, int sticky)
     }
 }
 
+
+
 /*
  * process_module calls us back here so we do the actual checkout stuff
  */
 /* ARGSUSED */
 static int
-checkout_proc (int argc, char **argv, char *where_orig, char *mwhere, char *mfile, int shorten, int local_specified, char *omodule, char *msg)
+checkout_proc (int argc, char **argv, char *where_orig, char *mwhere,
+	       char *mfile, int shorten, int local_specified, char *omodule,
+	       char *msg)
 {
     char *myargv[2];
     int err = 0;
@@ -586,17 +594,14 @@ checkout_proc (int argc, char **argv, char *where_orig, char *mwhere, char *mfil
     char *oldupdate = NULL;
     char *where;
 
-    if (trace)
-	(void) fprintf (stderr,
-	                "%s-> checkout_proc (%s, %s, %s, %d, %d, %s, %s)\n",
-			CLIENT_SERVER_STR,
-			where_orig ? where_orig : "(null)",
-			mwhere ? mwhere : "(null)",
-			mfile ? mfile : "(null)",
-	                shorten, local_specified,
-			omodule ? omodule : "(null)",
-			msg ? msg : "(null)"
-	               );
+    TRACE (TRACE_FUNCTION, "checkout_proc (%s, %s, %s, %d, %d, %s, %s)\n",
+	   where_orig ? where_orig : "(null)",
+	   mwhere ? mwhere : "(null)",
+	   mfile ? mfile : "(null)",
+	   shorten, local_specified,
+	   omodule ? omodule : "(null)",
+	   msg ? msg : "(null)"
+	  );
 
     /*
      * OK, so we're doing the checkout! Our args are as follows: 
@@ -800,7 +805,7 @@ internal error: %s doesn't start with %s in checkout_proc",
 
 	/* We always create at least one directory, which corresponds to
 	   the entire strings for WHERE and REPOSITORY.  */
-	head = (struct dir_to_build *) xmalloc (sizeof (struct dir_to_build));
+	head = xmalloc (sizeof (struct dir_to_build));
 	/* Special marker to indicate that we don't want build_dirs_and_chdir
 	   to create the CVSADM directory for us.  */
 	head->repository = NULL;
@@ -823,8 +828,7 @@ internal error: %s doesn't start with %s in checkout_proc",
 	    if (cp == NULL)
 		break;		/* we're done */
 
-	    new = (struct dir_to_build *)
-		xmalloc (sizeof (struct dir_to_build));
+	    new = xmalloc (sizeof (struct dir_to_build));
 	    new->dirpath = xmalloc (strlen (where));
 
 	    /* If the user specified an absolute path for where, the
@@ -912,7 +916,8 @@ internal error: %s doesn't start with %s in checkout_proc",
 	   build_one_dir whenever the -d command option was specified
 	   to checkout.  */
 
-	if (!isabsolute (where) && top_level_admin && m_type == CHECKOUT)
+	if (!isabsolute (where) && config->top_level_admin
+	    && m_type == CHECKOUT)
 	{
 	    /* It may be argued that we shouldn't set any sticky
 	       bits for the top-level repository.  FIXME?  */
@@ -957,11 +962,10 @@ internal error: %s doesn't start with %s in checkout_proc",
 		    error (1, 0, "there is no repository %s", repository);
 
 		Create_Admin (".", preload_update_dir, repository,
-			      (char *) NULL, (char *) NULL, 0, 0,
-			      m_type == CHECKOUT);
+			      NULL, NULL, 0, 0, m_type == CHECKOUT);
 		fp = open_file (CVSADM_ENTSTAT, "w+");
-		if (fclose(fp) == EOF)
-		    error(1, errno, "cannot close %s", CVSADM_ENTSTAT);
+		if (fclose (fp) == EOF)
+		    error (1, errno, "cannot close %s", CVSADM_ENTSTAT);
 #ifdef SERVER_SUPPORT
 		if (server_active)
 		    server_set_entstat (where, repository);
@@ -992,13 +996,18 @@ internal error: %s doesn't start with %s in checkout_proc",
 		error (1, 0, "cannot export into working directory");
 
 	    /* get the contents of the previously existing repository */
-	    repos = Name_Repository ((char *) NULL, preload_update_dir);
+	    repos = Name_Repository (NULL, preload_update_dir);
 	    if (fncmp (repository, repos) != 0)
 	    {
+		char *prepos = xstrdup (primary_root_inverse_translate (repos));
+		char *prepository =
+		    xstrdup (primary_root_inverse_translate (repository));
 		error (0, 0, "existing repository %s does not match %s",
-		       repos, repository);
+		       prepos, prepository);
 		error (0, 0, "ignoring module %s", omodule);
 		free (repos);
+		free (prepos);
+		free (prepository);
 		err = 1;
 		goto out;
 	    }
@@ -1013,7 +1022,7 @@ internal error: %s doesn't start with %s in checkout_proc",
      */
     if (pipeout)
     {
-	if ( CVS_CHDIR (repository) < 0)
+	if (CVS_CHDIR (repository) < 0)
 	{
 	    error (0, errno, "cannot chdir to %s", repository);
 	    err = 1;
@@ -1022,7 +1031,7 @@ internal error: %s doesn't start with %s in checkout_proc",
 	which = W_REPOS;
 	if (tag != NULL && !tag_validated)
 	{
-	    tag_check_valid (tag, argc - 1, argv + 1, 0, aflag, NULL);
+	    tag_check_valid (tag, argc - 1, argv + 1, 0, aflag, NULL, false);
 	    tag_validated = 1;
 	}
     }
@@ -1032,7 +1041,7 @@ internal error: %s doesn't start with %s in checkout_proc",
 	if (tag != NULL && !tag_validated)
 	{
 	    tag_check_valid (tag, argc - 1, argv + 1, 0, aflag,
-			     repository);
+			     repository, false);
 	    tag_validated = 1;
 	}
     }
@@ -1061,12 +1070,12 @@ internal error: %s doesn't start with %s in checkout_proc",
 	if (!pipeout)
 	    history_write (m_type == CHECKOUT ? 'O' : 'E', preload_update_dir,
 			   history_name, where, repository);
-	err += do_update ( 0, (char **) NULL, options, tag, date,
-			   force_tag_match, 0 /* !local */ ,
-			   1 /* update -d */ , aflag, checkout_prune_dirs,
-			   pipeout, which, join_rev1, join_rev2,
-			   preload_update_dir, m_type == CHECKOUT,
-			   repository );
+	err += do_update (0, NULL, options, tag, date,
+			  force_tag_match, false /* !local */ ,
+			  true /* update -d */ , aflag, checkout_prune_dirs,
+			  pipeout, which, join_rev1, join_rev2,
+			  preload_update_dir, m_type == CHECKOUT,
+			  repository);
 	goto out;
     }
 
@@ -1103,7 +1112,7 @@ internal error: %s doesn't start with %s in checkout_proc",
 		Register (entries, finfo.file,
 			  vers->vn_rcs ? vers->vn_rcs : "0",
 			  line, vers->options, vers->tag,
-			  vers->date, (char *) 0);
+			  vers->date, NULL);
 		free (line);
 	    }
 	    freevers_ts (&vers);
@@ -1119,19 +1128,21 @@ internal error: %s doesn't start with %s in checkout_proc",
 		       repository);
 
     /* go ahead and call update now that everything is set */
-    err += do_update ( argc - 1, argv + 1, options, tag, date,
-		       force_tag_match, local_specified, 1 /* update -d */,
-		       aflag, checkout_prune_dirs, pipeout, which, join_rev1,
-		       join_rev2, preload_update_dir, m_type == CHECKOUT,
-		       repository );
+    err += do_update (argc - 1, argv + 1, options, tag, date,
+		      force_tag_match, local_specified, true /* update -d */,
+		      aflag, checkout_prune_dirs, pipeout, which, join_rev1,
+		      join_rev2, preload_update_dir, m_type == CHECKOUT,
+		      repository);
 out:
     free (preload_update_dir);
     preload_update_dir = oldupdate;
     free (where);
     free (repository);
-    return (err);
+    return err;
 }
 
+
+
 static char *
 findslash (char *start, char *p)
 {
@@ -1144,6 +1155,8 @@ findslash (char *start, char *p)
     return NULL;
 }
 
+
+
 /* Return a newly malloc'd string containing a pathname for CVSNULLREPOS,
    and make sure that it exists.  If there is an error creating the
    directory, give a fatal error.  Otherwise, the directory is guaranteed
@@ -1170,6 +1183,8 @@ emptydir_name (void)
     return repository;
 }
 
+
+
 /* Build all the dirs along the path to DIRS with CVS subdirs with appropriate
  * repositories.  If DIRS->repository is NULL or the directory already exists,
  * do not create a CVSADM directory for that subdirectory; just CVS_CHDIR into
