@@ -1,18 +1,16 @@
 /* Interface between the client and the rest of CVS.  */
 
 /* Stuff shared with the server.  */
-extern char *mode_to_string (mode_t);
-extern int change_mode (char *, char *, int);
+char *mode_to_string (mode_t);
+int change_mode (const char *, const char *, int);
 
 extern int gzip_level;
 extern int file_gzip_level;
 
 struct buffer;
 
-void make_bufs_from_fds ( int, int, int,
-			        struct buffer **,
-			        struct buffer **,
-			        int );
+void make_bufs_from_fds (int, int, int, cvsroot_t *,
+			 struct buffer **, struct buffer **, int);
 
 
 #if defined (CLIENT_SUPPORT) || defined (SERVER_SUPPORT)
@@ -72,16 +70,19 @@ void send_to_server (const char *str, size_t len);
 void read_from_server (char *buf, size_t len);
 
 /* Internal functions that handle client communication to server, etc.  */
-int supported_request (char *);
-void option_with_arg (char *option, char *arg);
+bool supported_request (const char *);
+void option_with_arg (const char *option, const char *arg);
 
 /* Get the responses and then close the connection.  */
-extern int get_responses_and_close (void);
+int get_responses_and_close (void);
 
-extern int get_server_responses (void);
+int get_server_responses (void);
 
 /* Start up the connection to the server on the other end.  */
 void
+open_connection_to_server (cvsroot_t *root, struct buffer **to_server_p,
+                           struct buffer **from_server_p);
+void
 start_server (void);
 
 /* Send the names of all the argument files to the server.  */
@@ -109,13 +110,13 @@ send_files (int argc, char **argv, int local, int aflag,
 
 /* Send an argument to the remote server.  */
 void
-send_arg (char *string);
+send_arg (const char *string);
 
 /* Send a string of single-char options to the remote server, one by one.  */
 void
-send_option_string (char *string);
+send_option_string (const char *string);
 
-extern void send_a_repository (const char *, const char *, const char *);
+void send_a_repository (const char *, const char *, const char *);
 
 #endif /* CLIENT_SUPPORT */
 
@@ -127,7 +128,7 @@ extern void send_a_repository (const char *, const char *, const char *);
 struct response
 {
     /* Name of the response.  */
-    char *name;
+    const char *name;
 
 #ifdef CLIENT_SUPPORT
     /*
@@ -137,14 +138,15 @@ struct response
      * Note that although LEN is given, ARGS is also guaranteed to be
      * '\0' terminated.
      */
-    void (*func) (char *args, int len);
+    void (*func) (char *args, size_t len);
 
     /*
      * ok and error are special; they indicate we are at the end of the
      * responses, and error indicates we should exit with nonzero
      * exitstatus.
      */
-    enum {response_type_normal, response_type_ok, response_type_error} type;
+    enum {response_type_normal, response_type_ok, response_type_error,
+	  response_type_redirect} type;
 #endif
 
     /* Used by the server to indicate whether response is supported by
@@ -176,22 +178,21 @@ extern struct response responses[];
 
 #ifdef CLIENT_SUPPORT
 
-extern void client_senddate (const char *date);
-extern void client_expand_modules (int argc, char **argv, int local);
-extern void client_send_expansions (int local, char *where,
-					  int build_dirs);
-extern void client_nonexpanded_setup (void);
+void client_senddate (const char *date);
+void client_expand_modules (int argc, char **argv, int local);
+void client_send_expansions (int local, char *where, int build_dirs);
+void client_nonexpanded_setup (void);
 
-extern void send_init_command (void);
+void send_init_command (void);
 
 extern char **failed_patches;
 extern int failed_patches_count;
 extern char *toplevel_wd;
-extern void client_import_setup (char *repository);
-extern int client_process_import_file
+void client_import_setup (char *repository);
+int client_process_import_file
     (char *message, char *vfile, char *vtag, int targc, char *targv[],
      char *repository, int all_files_binary, int modtime);
-extern void client_import_done (void);
-extern void client_notify (const char *, const char *, const char *, int,
-                           const char *);
+void client_import_done (void);
+void client_notify (const char *, const char *, const char *, int,
+                    const char *);
 #endif /* CLIENT_SUPPORT */
