@@ -324,7 +324,7 @@ ignore_directory (const char *name)
     i = dir_ign_current;
     while (i--)
     {
-	if (strncmp (name, dir_ign_list[i], strlen (dir_ign_list[i])) == 0)
+	if (strncmp (name, dir_ign_list[i], strlen (dir_ign_list[i])+1) == 0)
 	    return 1;
     }
 
