@@ -22,21 +22,8 @@
 #include <io.h>
 #include <fcntl.h>
 
-#ifdef HAVE_VPRINTF
-#if defined (USE_PROTOTYPES) ? USE_PROTOTYPES : defined (__STDC__)
-#include <stdarg.h>
-#define VA_START(args, lastarg) va_start(args, lastarg)
-#else
-#include <varargs.h>
-#define VA_START(args, lastarg) va_start(args)
-#endif
-#else
-#define va_alist a1, a2, a3, a4, a5, a6, a7, a8
-#define va_dcl char *a1, *a2, *a3, *a4, *a5, *a6, *a7, *a8;
-#endif
-
-static void run_add_arg PROTO((const char *s));
-static void run_init_prog PROTO((void));
+static void run_add_arg( const char *s );
+static void run_init_prog( void );
 
 extern char *strtok ();
 
@@ -53,29 +40,17 @@ extern char *strtok ();
  * The execvp() syscall will be used, so that the PATH is searched correctly.
  * File redirections can be performed in the call to run_exec().
  */
-static char *run_prog;
 static char **run_argv;
 static int run_argc;
 static int run_argc_allocated;
 
-/* VARARGS */
-#if defined (HAVE_VPRINTF) && (defined (USE_PROTOTYPES) ? USE_PROTOTYPES : defined (__STDC__))
-void 
-run_setup (const char *fmt,...)
-#else
-void 
-run_setup (fmt, va_alist)
-    char *fmt;
-    va_dcl
-#endif
+void
+run_setup (const char *prog)
 {
-#ifdef HAVE_VPRINTF
-    va_list args;
-#endif
     char *cp;
     int i;
 
-    run_init_prog ();
+    char *run_prog;
 
     /* clean out any malloc'ed values from run_argv */
     for (i = 0; i < run_argc; i++)
@@ -88,18 +63,13 @@ run_setup (fmt, va_alist)
     }
     run_argc = 0;
 
-    /* process the varargs into run_prog */
-#ifdef HAVE_VPRINTF
-    VA_START (args, fmt);
-    (void) vsprintf (run_prog, fmt, args);
-    va_end (args);
-#else
-    (void) sprintf (run_prog, fmt, a1, a2, a3, a4, a5, a6, a7, a8);
-#endif
+    run_prog = xstrdup (prog);
 
     /* put each word into run_argv, allocating it as we go */
     for (cp = strtok (run_prog, " \t"); cp; cp = strtok ((char *) NULL, " \t"))
 	run_add_arg (cp);
+
+    free (run_prog);
 }
 
 void
@@ -159,20 +129,11 @@ run_add_arg (s)
 	run_argv[run_argc] = (char *) 0;	/* not post-incremented on purpose! */
 }
 
-static void
-run_init_prog ()
-{
-    /* make sure that run_prog is allocated once */
-    if (run_prog == (char *) 0)
-	run_prog = xmalloc (10 * 1024);	/* 10K of args for _setup and _arg */
-}
-
-
 int
 run_exec (stin, stout, sterr, flags)
-    char *stin;
-    char *stout;
-    char *sterr;
+    const char *stin;
+    const char *stout;
+    const char *sterr;
     int flags;
 {
     int shin, shout, sherr;
@@ -563,7 +524,7 @@ start_child (char *command, HANDLE in, HANDLE out)
    construct a command line that one might pass to CreateProcess.
    Try to quote things appropriately.  */
 static char *
-build_command (char **argv)
+build_command (char *const *argv)
 {
     int len;
 
@@ -636,7 +597,7 @@ build_command (char **argv)
    Return the handle of the child process (this is what
    _cwait and waitpid expect).  */
 int
-piped_child (char **argv, int *to, int *from)
+piped_child (char *const *argv, int *to, int *from)
 {
   int child;
   HANDLE pipein[2], pipeout[2];
@@ -644,7 +605,7 @@ piped_child (char **argv, int *to, int *from)
 
   /* Turn argv into a form acceptable to CreateProcess.  */
   command = build_command (argv);
-  if (! command)
+  if (!command)
       return -1;
 
   /* Create pipes for communicating with child.  Arrange for
